function [invdist,exitflag] = invdist(P,id,tol,maxiter,startDist)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: calculate the invariant distribution of an finite state markov
%          chain with transition probability P
%       where
%      "P" is transition matrix of S by S (P(i,j)=prob(t+1=j|t=i))
%      "id"(optional) is method for calculation: "default =1" 
%            1 = eigenvector associated with unit eigenvalue;
%            2 = iteration method based on convergence theorem
%      "tol"(optional) is tolerance level: "default = max((1e-6)/size(P,1),1e-11)"
%      "maxiter"(optional) is the max number of iteration for id = 2: "default = max(800,200*size(P,1))"
%      "startDist"(optional) is S by S starting vector of invariant
%                       distribution: "default = uniform distribution"
%
%      "invdist" is S by N vector of invariant distributions, where N is
%        the number of "basis vector" of invariant distributions(see below)
%      "exitflag"(optional) is the condition for exit 
%            1 = iteration succeeded;
%            0 = the maxium number of iteration exceeded
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Note:(a) If the function returns multiple invariant distribution, every
%          convex combination of them will also be invariant distribution.
%          This follows from the definitions of invariant distributions.
%      (b) "id=1" works especially well for large sparse matrix (use sparse(P) when calling the function); 
%          "id=2" works for small or large dimension matrix, but could be hundreds of times slower for large sparse matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Jinhui Bai(jinhui.bai@gmail.com), March 2, 2004; 
% Revised: Oct 10,2004; Sept 28, 2005; October 20, 2005
% Revised: July 16,2020: 
%          changed eigenvalue algorithm to use a sigma parameter "eigs(P.',1,1 - 1e-12)"  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% input default value
if nargin == 1
    id = 1;
    tol = max((1e-6)/size(P,1),1e-11);
    maxiter = max(800,200*size(P,1));
    startDist = (1/size(P,1))*ones(size(P,1),1);
elseif nargin == 2
    tol = max((1e-6)/size(P,1),1e-11);
    maxiter = max(800,200*size(P,1));
    startDist = (1/size(P,1))*ones(size(P,1),1);
elseif nargin == 3
    maxiter = max(800,200*size(P,1));
    startDist = (1/size(P,1))*ones(size(P,1),1);
elseif nargin == 4
    startDist = (1/size(P,1))*ones(size(P,1),1);    
end

% check whether P is a square matrix
[nrow,ncolumn]=size(P);
if nrow ~= ncolumn
    error('transition matrix must be square');
end

% check transition makes a probability matrix
sum_P = sum(P,2);
index_P = all(sum_P >= (ones(nrow,1) - 1e-8)); 
    % Note: The line above functions as "isequal(sum_P,ones(nrow,1))". But it is
    % more robust to the roundoff error.

if ~index_P
    error('some row do not sum to one');
end

% check "startDist" has the right properties
if (nargin == 5)
    if (size(startDist,1) ~= size(P,1))
        error('starting distribution must have the same size as transition probability');
    end
    if (size(startDist,2) ~= 1)
        error('starting distribution must be a column vector');
    end
    if (sum(startDist) <= (1 - 1e-8)) || (sum(startDist) >= (1 + 1e-8))
        error('starting distribution must be a probability vector');
    end
end

%
% Calculate invariant distribution
%
if isequal(id,1)
    % calculate eigenvalue and find the unit eigenvalue and eigenvector
    sigma = 1.0 - 1e-12; % a reference point close to 1: note it cannot be 1, which causes singularity
    [v,lambda] = eigs(P.',1,sigma); % find eigen of P transpose cloest to sigma
    test = abs(lambda - 1.0);      % find the index for unit eigenvalue
    % check the uniqueness of invariant distribution
    if (test > tol)
        error('It contradicts theory: there is no invariant distribution??');
    end
    % calculate invariant distribution
    invdist = v./sum(v,1); % normalize eigenvector, which is base vector of invariant dist            
elseif isequal(id,2)
    invdist = startDist;        % uniform dist as starting guess
    test = 1.0;                   % metric for convergence
    niter = 1;                  % number of iteration
    while ((niter <= maxiter) && (test > tol))
        probst = P.'*invdist;                % iteration
        test=max(abs(probst-invdist));
        invdist = probst;
        niter = niter + 1;
    end
else
    error('id must be either 1 or 2');
end

% output exitflag
if nargout == 2
    if (test <= tol)
        exitflag = 1;
    else
        exitflag = 0;
    end
end
