%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   MATLAB Code for Computing an Aiyagari Model with Exogenous Labor 
%                                            Jinhui Bai
%                                   First Version: 07/17/2020
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                               Note
%  This code is mainly written for the teaching of PhD Numerical Methods in Economics. It illustrates how equilibrium  
%  computation works in a concrete example. It aims at readability rather than efficiency. An experienced programmer 
%  may want to program somewhat differently to improve the code efficiency in terms of speed, memory, etc. 
%
%  This code is written and tested in MATLAB R2017a
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; close all; clc; % clear the workspace
time_start = tic; % start recording time

fprintf('\n********************************************************************************\n');
fprintf('** Solve an Aiyagari Model with Exogenous Labor: Continuous State Method\n');
fprintf('**********************************************************************************\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Use an Equation Solver to Solve the Equilibrium Capital Stock
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x0 = 12.186974388012741; % initial capital stock
% options.Algorithm = 'Newton';'Broyden';'BroydenInv';'Cauchy';'FixedJacobian'
optRoot = struct('Algorithm','BroydenInv','NormOrder',Inf,'Display','iter',...
                 'MaxIter',100,'MaxFunEvals',400,'TolFun',1e-4,'TolX',sqrt(eps),'Jacobian','off','FinDiffType','forward',...                           
                 'FinDiffRelStep',sqrt(eps),'LineSearchMethod','Armijo','InitStepSize',1.0,'MaxLineSearch',10,'Restarts',true,...
                 'ScaleProblem','None','TypicalX',[],'JacobianInv0',0.001*eye(length(x0)),'Jacobian0',1000*eye(length(x0)));
% call solver
[x,fval,exitflag,output] = myfsolve(@compute_IF1_CS,x0,optRoot); % Call solver

% Report Time
time_spent = toc(time_start); % record time    
fprintf(1,'Time Spent in Equilibrium Computation: %14.4g Seconds\n',time_spent);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local Functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local Functions: Equilibrium Loop to Solve an Income Fluctuation Problem
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Excess_Demand = compute_IF1_CS(AssetMean)
% AssetMean = 12.187277733295684;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               Compute an Income Fluctuation Problem
%                       Table of Contents
%  1. Set up model parameters
%  2. Solve the problem through Dynamic Programming
%  3. Compute the invariant distribution of the joint state variables
%  4. Compute the Aggregate Asset Holdings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
time_start = tic; % start recording time

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Set up the model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 1.1 Parameters of the economy
%
% Preference
sigma  = 1.00;                % risk aversion              
beta   = 0.99;                % subjective discount factor

% production function
TFP    = 1.00;                % TFP                      
alpha  = 0.36;                % capital share              
delta  = 0.025;               % capital depreciation rate
labor  = 0.3333;              % exogenous labor supply

% exogenous shock
shock   = [0.10;1.00];         % the grid of the exogenous shock
prob   = [0.3333 0.6667;
    	  0.0300 0.9700];     % Prob Transition Matrix: prob(i,j) = p(s(t+1)=sj|s(t)=si)
nShock = size(prob,1);        % grid number of shocks

invDistShock = invdist(prob);  % invariant distribution of idiosyncratic shocks
laborMean = invDistShock.'*shock*labor; % average labor efficiency

%
% 1.2 Calculate Market Prices for the economy
%
r = TFP*alpha*(AssetMean.^(alpha-1.0))*(laborMean.^(1-alpha)) - delta;        % the real interest rate
wage = TFP*(1.0 - alpha)*(AssetMean.^alpha)*(laborMean.^(-alpha));    % the real wage rate

fprintf(1,'\n** The Asset Mean is %12.8g, the interest rate is %12.8g, the wage rate is %12.8g\n',AssetMean,r,wage);

%
% 1.3 Form asset grid for dynamic programming problem
%
% Note: a good reference point is the deterministic steady state 
agridMin = 0.0;                       % minimum value of asset grid
agridMax = 55.0;                       % maximum value of asset grid
aCutPoint1 = 1.0;                      % cutoff point
nAsset1 =  51;                       % number of grid points: first cutoff
incast1 = (aCutPoint1-agridMin)/(nAsset1-1); % size of asset grid increments
aCutPoint2 = 28;                      % cutoff point
nAsset2 =  100;                       % number of grid points: first cutoff
incast2 = (aCutPoint2-aCutPoint1)/(nAsset2); % size of asset grid increments
nAsset3 = 100;                               % number of grid points: second cutoff
incast3 = (agridMax-aCutPoint2)/(nAsset3);   % size of asset grid increments
agrid = [agridMin:incast1:aCutPoint1,aCutPoint1+incast2:incast2:aCutPoint2,aCutPoint2+incast3:incast3:agridMax]';    % asset grid
nAsset = nAsset1 + nAsset2 + nAsset3;

nTotalGrid = nAsset*nShock;           % total number of grid used

% Vectorized Grids
agrid_vec = repmat(agrid,[nShock 1]);   % asset grid in combined states
shock_vec = kron(shock,ones(nAsset,1));   % shock grid in combined states
prob_mat = kron(prob,ones(nAsset,1));

% Set up the parameters for the optimization problem
total_vec = (1 + r)*agrid_vec + wage*labor*shock_vec; % total resource available
lb_vec = repmat(agridMin,[nTotalGrid,1]); %#ok lower bound
ub_vec = min(total_vec,agridMax); % upper bound
optOptim = optimset('Display','off','TolX',1.5e-8,'MaxIter',100); % set control parameters

%
% 1.4 Form asset grid for Invariant Distribution
%
nAssetInvDist = 5001;
incastInvDist = (agridMax-agridMin)/(nAssetInvDist-1);    % size of asset grid increments
agridInvDist = (agridMin:incastInvDist:agridMax).';       % asset grid

nTotalGridInvDist = nAssetInvDist*nShock;          % total number of grid used

% Vectorize Grids
agridInvDist_vec = repmat(agridInvDist,[nShock 1]);   % asset grid in combined states
techInvDist_vec = kron(shock,ones(nAssetInvDist,1));   % tech grid in combined states
probInvDist_mat = kron(prob,ones(nAssetInvDist,1));   % prob transition in combined states

totalInvDist_vec = (1 + r)*agridInvDist_vec + wage*labor*techInvDist_vec; % total resource available
lbInvDist_vec = repmat(agridMin,[nTotalGridInvDist,1]); %#ok lower bound
ubInvDist_vec = min(totalInvDist_vec,agridMax); % upper bound

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. Household Dynamic Programming (by Continuous State Methods)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Two general methods for solving household DP problem: standard Value Function Iteration or Endogenous Grid Method (EGM)
% For EGM, we implemented four variations depending on the choices of function approximations: 
% (a) 'EGM_p': policy function (through Euler Equation); 
% (b) 'EGM_vDer': derivative of value function (through FOC and Envelope Theorem, and is a transformation of Euler Equation); 
% (c) 'EGM_v_vDer': value function and its derivative (through Bellman equation, Envelope Theorem and Cubic Hermite Interpolation)
% (d) 'EGM_v': value function (through Bellman equation)

DP_Algorithm = 'VFI'; % 'VFI';'EGM_p';'EGM_vDer';'EGM_v_vDer';'EGM_v';

%
% 2.1 Initialize some variables
%
maxiter_v = 2000;                % maximum iteration for value fcn iteration
maxiter_h = 500;                 % maximum iteration for Modified Howard Improvement 

tol_v = 1e-6;                     % tolerance for cnge of value function
tol_p = 1e-8;                     % tolerance for cnge of policy function
tol_hv = 1e-10;                    % tolerance for cnge of Howard Step
tol_hp = 1e-4;                    % tolerance for starting Howard Step

v    = zeros(nAsset,nShock);   % value function
vDer = zeros(nAsset,nShock);   % Derivative of value function
dec  = zeros(nAsset,nShock);   % policy function

tv    = zeros(nAsset,nShock);   % new value function
tvDer = zeros(nAsset,nShock);   % Derivative of new value function
tdec  = zeros(nAsset,nShock);   % new policy function

v   = reshape(log(total_vec),size(v));   % value function for the last period
vDer = reshape((1+r)./total_vec,size(vDer));   % Derivative of value function for the last period

% v   = reshape(log(total_vec - agrid_vec),size(v))/(1-beta);   % value function for the last period
% vDer = reshape((1+r)./(total_vec - agrid_vec),size(vDer));   % Derivative of value function for the last period

%
% 2.2 Solve DP Problem
%
fprintf(1,'\n********************************************************************************\n');
fprintf(1,'** Solve Households Dynamic Programming Problem\n');
fprintf(1,'**********************************************************************************\n');

fprintf(1,'%12s%12s%12s%12s%12s%12s\n','niter_v','metric_v','metric_p','metric_h','niter_h','metric_hv'); % Print out Iteration Header
time_start_VFI = tic; % start recording time

for niter_v = 1:maxiter_v
    if strcmp(DP_Algorithm,'VFI') % standard VFI
        vfn = spline(agrid,v.');   % interpolate value function using cubic splines with not-a-knot condition
        % Solve Bellman Equation Optimization Problem
        [tdec(:),tv(:),tvDer(:),exitflag_vec] = Bellman_solve(lb_vec,ub_vec,optOptim,total_vec,sigma,beta,prob_mat,vfn);%#ok
    elseif strncmp(DP_Algorithm,'EGM',3) % Endogenous Grid Method using Value Function Approximation
        eval_mat = repmat(agrid,[1 nShock]); % evaluation matrix at exogenous grid points
        [tdec,tv,tvDer] = EGM_solve(eval_mat,sigma,beta,prob_mat,agrid,vDer,v,r,wage,labor,shock,DP_Algorithm);
    else
        error('DP Algorithm is not correctly specified');
    end

    % calculating the change of value and policy function
    metric_v = max(abs(tv(:) - v(:))./(abs(v(:)) + 1.0));
    metric_p = max(abs(tdec(:) - dec(:))./(abs(dec(:)) + 1.0));
    
    % Check the convergence
    niter_h = 0; % initialized Howard Step Iteration Number
    metric_hv = NaN; % initialized Howard Step Value Function Changes
    metric_h = NaN; % initialized Howard Step Value Function Changes
    
    if (metric_v <= tol_v) && (metric_p <= tol_p) % converged: break out of the loop
        % print out the iteration results
        fprintf(1,'%12d%12.4g%12.4g%12.4g%12.4g%12.4g\n',[round(niter_v),metric_v,metric_p,metric_h,niter_h,metric_hv]);        
        break;
    elseif strcmp(DP_Algorithm,'VFI') && (metric_p <= tol_hp) && (niter_v >= 2) % policy converged (excluding 1st iter): VFI Howard Improvement        
        % (Modified) Howard Improvement Step
        v_h = tv; % intialize Howard Step Value Function
        for niter_hv = 1:maxiter_h
            vfn = spline(agrid,v_h.');   % interpolate value function using cubic splines with not-a-knot condition
%             vfn = pchip(agrid,v_h.');   % interpolate value function using cubic splines with not-a-knot condition            
            tv_h = bellmanMax_vec(tdec(:),total_vec,sigma,beta,prob_mat,vfn);
            % update metrics
            metric_hv = max(abs(tv_h - v_h(:))./(abs(v_h(:)) + 1.0)); % calculate change within Howard Step
            niter_h = niter_hv; % update iteration number
            v_h(:) = tv_h;
            % check convergence and update
            if metric_hv <= tol_hv
                break;
            end
        end
        metric_h = max(abs(v_h(:) - v(:))./(abs(v(:)) + 1.0)); % calculate change across Two Howard Steps
        v(:) = v_h(:);            % update value function        
    else % stick to fixed point iteration        
        v = tv;            % update value function
        vDer = tvDer;      % update value function derivative
    end
    % print out the iteration results
    fprintf(1,'%12d%12.4g%12.4g%12.4g%12.4g%12.4g\n',[round(niter_v),metric_v,metric_p,metric_h,niter_h,metric_hv]); 
    
    % update other parameters
    dec = tdec;        % update policy function
end

time_spent_VFI = toc(time_start_VFI); % record time
fprintf(1,'\nTime Spent in Computing VFI: %14.4g Seconds\n',time_spent_VFI);

%
% 2.3 Resolve the household problem with a finer grid
%
time_start_DPInvDist = tic; % start recording time

if strcmp(DP_Algorithm,'VFI')
    [tdecInvDist,tvInvDist,tvDerInvDist,exitflagInvDist_vec] = Bellman_solve(lbInvDist_vec,ubInvDist_vec,optOptim,...
                                                                totalInvDist_vec,sigma,beta,probInvDist_mat,vfn);%#ok
    decInvDist = reshape(tdecInvDist,nAssetInvDist,nShock);
elseif strncmp(DP_Algorithm,'EGM',3)
    evalInvDist_mat = repmat(agridInvDist,[1 nShock]); % evaluation matrix at exogenous grid points
    [tdecInvDist,tvInvDist,tvDerInvDist] = EGM_solve(evalInvDist_mat,sigma,beta,prob_mat,agrid,vDer,v,r,wage,labor,shock,DP_Algorithm);%#ok
    decInvDist = tdecInvDist;
end

time_spent_DPInvDist = toc(time_start_DPInvDist); % record time
fprintf(1,'\nTime Spent in Resolve Households Problem in Invariant Distribution: %14.4g Seconds\n',time_spent_DPInvDist);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3. Invariant distribution step
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% transition matrix for all grids
trans = transmat1d(prob,decInvDist,agridInvDist); % form joint transition matrix from Discrete Histogram Method

% invariant distibution for all grids
time_start_InvDist = tic; % start recording time

[dist_vec,exitflag_InvDist] = invdist(sparse(trans));          %#ok compute invariant distribution

time_spent_InvDist = toc(time_start_InvDist); % record time
fprintf(1,'\nTime Spent in Computing Invariant Distribution: %14.4g Seconds\n',time_spent_InvDist);

% invariant distribution: joint and marginal distribution in histogram
dist_2d = reshape(dist_vec,[nAssetInvDist nShock]); % the invariant distribution as a 2-dimensional joint distribution
dist_agridInvDist = sum(dist_2d,2); % marginal distribution over asset
dist_shock = (sum(dist_2d)).'; % marginal distribution over shocks

% invariant distribution: joint and marginal distribution in density function
density_2d = dist_2d(2:end,:)./sum(dist_2d(2:end,:));
density_2d = density_2d./(agridInvDist(2:end) - agridInvDist(1:(end-1)));

density_agridInvDist = dist_agridInvDist(2:end,:)./sum(dist_agridInvDist(2:end,:));
density_agridInvDist = density_agridInvDist./(agridInvDist(2:end) - agridInvDist(1:(end-1)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 4. Calculate the change in Aggregate Capital
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

AssetMeanNew = dist_agridInvDist.'*agridInvDist; 
Excess_Demand = AssetMean - AssetMeanNew; % Convert Fixed Point Problem to Root Finding Problem

%
% Game over: how fast is my job?
%
time_spent = toc(time_start); % record time    
fprintf(1,'\nTime Spent in Computation: %14.4g Seconds\n',time_spent);

% plot the result
figure(1)
plot(agrid,v);
title('Value Function');
xlabel('Asset');
ylabel('Value Function');

figure(2)
plot(agrid,dec,agrid,agrid,'--');
title('Policy Function');
xlabel('Asset');
ylabel('Policy Function');

figure(3)
bar(shock,dist_shock);
title('Histogram of Marginal Invariant Distribution: Shock');
xlabel('Asset');
ylabel('Histogram');

figure(4)
plot(agridInvDist,dist_agridInvDist);
title('Histogram of Marginal Invariant Distribution: Asset');
xlabel('Asset');
ylabel('Histogram');

figure(5)
plot(agridInvDist,dist_2d);
title('Histogram of Joint Invariant Distribution: Asset');
xlabel('Asset');
ylabel('Histogram');

figure(6)
plot(agridInvDist(2:end),density_agridInvDist);
title('Density of Marginal Invariant Distribution: Asset');
xlabel('Asset');
ylabel('Density');

figure(7)
plot(agridInvDist(2:end),density_2d);
title('Density of Joint Invariant Distribution: Asset');
xlabel('Asset');
ylabel('Density');

% save('Workspace_IF1_CSVFI.mat');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local Function: Solve Bellman Equation 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [tdec_vec,tv_vec,tvDer_vec,exitflag_vec] = Bellman_solve(lb_vec,ub_vec,optOptim,total_vec,sigma,beta,prob_mat,vfn)

% solve the Bellman Equation Optimization Problem
[tdec_vec,tv_vec,exitflag_vec] = goldenx(@bellmanMax_vec,lb_vec,ub_vec,optOptim.MaxIter,optOptim.TolX,total_vec,sigma,beta,prob_mat,vfn);
tvDer_vec = NaN(size(tv_vec)); % no derivative information needed in VFI: set to NaN


end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local Function: Endogenous Grid Method: Locate, Interpolate at Endogenous Grid Points, and Evaluate 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [tdec,tv,tvDer] = EGM_solve(eval_mat,sigma,beta,prob_mat,agrid,vDer,v,r,wage,labor,tech,flag)

% find all endogenous grid points for today's shock plus tomorrow's grid
nAsset = size(vDer,1);
nShock = size(vDer,2);

tdecEGM_mat = repmat(agrid,[1,nShock]); % interior choice on Endogenous Grid
expect_vDer_mat = reshape(sum(prob_mat.*repmat(vDer,[nShock,1]),2),[nAsset nShock]); % expected value of derivative
totalEGM_mat = (beta*expect_vDer_mat).^(-1./sigma) + tdecEGM_mat; % interior total resources on Endogenous Grid
agridEGM_mat = (totalEGM_mat - wage*labor*(tech(:).'))./(1+r); % interior Endogenous Grid
probEGM_Array = permute(reshape(prob_mat,[nAsset,nShock,nShock]),[1 3 2]); % prob with today's shock on 3rd dim

% loop through to evaluate at the given evaluation points
nEval = size(eval_mat,1);
tv    = zeros(nEval,nShock);   % new value function
tvDer = zeros(nEval,nShock);   % Derivative of new value function
tdec  = zeros(nEval,nShock);   % new policy function

for ii = 1:nShock
    % check and adjust constraint violation and missing corner choices
    if agridEGM_mat(1,ii) > agrid(1) % missing grid points around the flat part: add
        % locate and calculate points to add to the flat part
        idx_flat = (agrid < agridEGM_mat(1,ii));
        agridEGMTempAdd = agrid(idx_flat);
        tdecEGMTempAdd = repmat(agrid(1),[sum(idx_flat),1]);
        totalEGMTempAdd = (1+r)*agridEGMTempAdd + wage*labor*tech(ii);
        probEGMTempAdd = repmat(probEGM_Array(1,:,ii),[sum(idx_flat),1]);
        contvalueEGMTempAdd = repmat(v(1,:),[sum(idx_flat),1]);
        % add the additional points to restore the flat part
        agridEGMTemp = [agridEGMTempAdd;agridEGM_mat(:,ii)];
        tdecEGMTemp = [tdecEGMTempAdd;tdecEGM_mat(:,ii)];
        totalEGMTemp = [totalEGMTempAdd;totalEGM_mat(:,ii)];
        probEGMTemp = [probEGMTempAdd;probEGM_Array(:,:,ii)];
        contvalueEGMTemp = [contvalueEGMTempAdd;v];
    elseif agridEGM_mat(1,ii) < agrid(1) % some grid points violating borrowing constraint: drop
        idx_constraint = (agridEGM_mat(:,ii) >= agrid(1));
        agridEGMTemp = agridEGM_mat(idx_constraint,ii);
        tdecEGMTemp = tdecEGM_mat(idx_constraint,ii);
        totalEGMTemp = totalEGM_mat(idx_constraint,ii);
        probEGMTemp = probEGM_Array(idx_constraint,:,ii);
        contvalueEGMTemp = v(idx_constraint,:);
    else % no need for adjustment
        % select endogenous grids for today's shock ii
        agridEGMTemp = agridEGM_mat(:,ii);
        tdecEGMTemp = tdecEGM_mat(:,ii);
        totalEGMTemp = totalEGM_mat(:,ii);
        probEGMTemp = probEGM_Array(:,:,ii);
        contvalueEGMTemp = v;
    end
    % interpolation and evaluation 
    evalTemp = eval_mat(:,ii); % evaluation data site
    totalTemp = (1+r).*evalTemp + wage*labor*tech(ii); % total resources available
    
    if strcmp(flag,'EGM_p')
        tdec(:,ii) = reshape(pchip(agridEGMTemp,tdecEGMTemp,evalTemp.'),[],1);   % interpolate policy function using cubic splines
        tdec(:,ii) = max(tdec(:,ii),agrid(1)); % enforce borrowing constraint
        tvDer(:,ii) = (1+r).*(totalTemp - tdec(:,ii)).^(-sigma); % use envelope theorem to get derivatives at the new point
    elseif strcmp(flag,'EGM_vDer')
        tvDerEGMTemp = (1+r).*(totalEGMTemp - tdecEGMTemp).^(-sigma); % calculate derivative at the endogenous grids
        tvDer(:,ii) = reshape(pchip(agridEGMTemp,tvDerEGMTemp.',evalTemp.'),[],1);   % interpolate value derivative using cubic splines
        tvDer(:,ii) = max(tvDer(:,ii),(1+r)*(totalTemp - agrid(1)).^(-sigma)); % enforce min derivative at borrowing constraint
        % use envelope theorem to give an implicit decision at the interpolation points        
        tdec(:,ii) = totalTemp - (tvDer(:,ii)./(1+r)).^(-1/sigma); % find policy function from Envelope Theorem        
    elseif strcmp(flag,'EGM_v') || strcmp(flag,'EGM_v_vDer')
        % calculate new value function at the endogenous grid points
        tvEGMTemp = bellmanMax_vec(tdecEGMTemp,totalEGMTemp,sigma,beta,probEGMTemp,contvalueEGMTemp);
        if strcmp(flag,'EGM_v') % use value function information only
            vfnEGMTemp = spline(agridEGMTemp,tvEGMTemp.');   % interpolate value function using cubic splines
        elseif strcmp(flag,'EGM_v_vDer') % use value function and its derivative to interpolate Cubic Hermite Splines
            tvDerEGMTemp = (1+r).*(totalEGMTemp - tdecEGMTemp).^(-sigma);
            vfnEGMTemp = pwch(agridEGMTemp,tvEGMTemp.',tvDerEGMTemp.');   % interpolate value function using cubic splines            
        end
        vfnDerEGMTemp = fnder(vfnEGMTemp,1);    % the derivative of spline
        % evaluate at the exogenous grid
        tv(:,ii) = reshape(ppval(vfnEGMTemp,evalTemp.'),[],1);
        tvDer(:,ii) = reshape(ppval(vfnDerEGMTemp,evalTemp.'),[],1);
        tvDer(:,ii) = max(tvDer(:,ii),(1+r)*(totalTemp - agrid(1)).^(-sigma)); % enforce min derivative at borrowing constraint
        % use envelope theorem to give an implicit decision at the interpolation points        
        tdec(:,ii) = totalTemp - (tvDer(:,ii)./(1+r)).^(-1/sigma); % find policy function from Envelope Theorem
    end
end

end % end the local function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local Function: Vector Bellman equation function for max problem
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function tv_vec = bellmanMax_vec(aprime_vec,total_vec,sigma,beta,prob_mat,vfn)

% Consumption
cons_vec = total_vec - aprime_vec;

% Index for positive consumption
isvalid_vec = find(cons_vec > 0);

% Flow value for CRRA
util_vec = - repmat(realmax,size(cons_vec)); % give default flow utility minus infinity
if abs(sigma - 1.0) < eps % log utility
    util_vec(isvalid_vec) = log(cons_vec(isvalid_vec)); % update the util 
else % non-log CRRA utility
    util_vec(isvalid_vec) = ((cons_vec(isvalid_vec)).^(1-sigma))./(1-sigma); % update the util 
end

% Continuation Value
if isa(vfn,'numeric') % a continuation value matrix
    contvalue_vec = vfn;
elseif isa(vfn,'struct') && strcmp(vfn.form,'pp') % a structure representing splines
    contvalue_vec = (ppval(vfn,aprime_vec.')).';
elseif isa(vfn,'struct') && strncmpi(vfn.form,'poly',4) % a structure representing splines
    contvalue_vec = (mypolyval1(vfn,aprime_vec.')).';
end

% Bellman value
tv_vec = util_vec + beta*sum(prob_mat.*contvalue_vec,2); % sum(.,2): summation by row

end % end the local function
