%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   MATLAB Code for Computing Real Business Cycle Models with Exogenous Labor 
%                   Algorithm: Bellman Equation Collocation with (Modified) Howard Improvement  
%                                            Jinhui Bai
%                                   First Version: 05/22/2020
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                               Note
%  This code is mainly written for the teaching of PhD Numerical Methods in Economics. It illustrates how equilibrium  
%  computation works in a concrete example. It aims at readability rather than efficiency. An experienced programmer 
%  may want to program somewhat differently to improve the code efficiency in terms of speed, memory, etc. 
%
%  This code is written and tested in MATLAB R2017a. The following Toolbox Functions are only used for derivative-based
%  method and can be commented out if you use derivative free algorithms and do not have these toolboxes installed.
%  (a) Curve Fitting Toolbox: fnder.m 
%  (b) Optimization Toolbox: fmincon.m; fsolve.m; 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                       Table of Contents
%  1. Set up model parameters
%  2. Solve the problem through Continuous State Value Function Iteration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; close all; clc; % clear the workspace
time_start = tic; % start recording time
fprintf('\n***********************************************************************************************************\n');
fprintf('** Solve a RBC Model with Exogenous Labor: Bellman Equation Collocation with (Modified) Howard Improvement\n');
fprintf('**************************************************************************************************************\n');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Set up the model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 1.1 Parameters of the economy
%
% Preference
sigma  = 1.00;                % risk aversion              
beta   = 0.99;                % subjective discount factor

% production function
alpha  = 0.36;                % capital share              
delta  = 0.025;               % capital depreciation rate
labor  = 0.3333;              % exogenous labor supply

% exogenous shock
tech   = [0.99;1.01];         % the grid of the exogenous shock
prob   = [0.8750 0.1250;
    	  0.1250 0.8750];     % Prob Transition Matrix: prob(i,j) = p(s(t+1)=sj|s(t)=si)
nShock = size(prob,1);        % grid number of shocks

%
% 1.2 Form asset grid
%
% Note: a good reference point is the deterministic steady state 
% aSS = (((1/beta - 1 + delta)/alpha).^(1/(alpha - 1)))*labor;
agridMin = 12.2;                       % minimum value of asset grid
agridMax = 13.0;                       % maximum value of asset grid
nAsset = 100;                          % grid number of asset
% a uniform grid
incast = (agridMax - agridMin)/(nAsset - 1); % size of asset grid increments
agrid = (agridMin:incast:agridMax).';    % asset grid

% Chebyshev Nodes
% polyInterval = [agridMin,agridMax];
% polyOrder = 9;
% polyform = 'polycheby'; % Chebyshev:'polycheby'; Monomial:'polymonomial'
% [agrid_unit,agrid] = chebynodes1(nAsset,polyInterval);

nTotalGrid = nAsset*nShock;           % total number of grid used

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. Continuous State Value Function Iteration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 2.1 Initialize some variables
%
maxiter_v = 2000;                 % maximum iteration for value fcn iteration
maxiter_h = 0;                  % maximum iteration for Modified Howard Improvement 

tol_v = 1e-6;                     % tolerance for cnge of value function
tol_p = 1e-6;                     % tolerance for cnge of policy function
tol_hv = tol_v;                   % tolerance for cnge of Howard Step
tol_hp = 1e-4;                    % tolerance for starting Howard Step

v   = zeros(nAsset,nShock);   % value function
dec = zeros(nAsset,nShock);   % policy function in numbers

% calculate flow utility for each combination of grid points: vectorize with order (1)agrid;(2)discrete shock;
agrid_vec = repmat(agrid,[nShock 1]);   % asset grid in combined states
tech_vec = kron(tech,ones(nAsset,1));   % tech grid in combined states
total_vec = tech_vec.*(agrid_vec.^alpha).*(labor.^(1 - alpha)) + (1 - delta).*agrid_vec; % total resource available

prob_mat = kron(prob,ones(nAsset,1));

% Set up the parameters for the optimization problem
% Input parameters
% lb_vec = repmat(agridMin,[nTotalGrid,1]); % 
lb_vec = zeros(nTotalGrid,1); % lower bound 
ub_vec = total_vec; % upper bound
x0_vec = (lb_vec + ub_vec)/2;
optim_fcn = 'fmincon'; % 'fminbnd';'fmincon';'fzero';'fsolve' 
Algorithm = 'sqp'; % 'sqp';'trust-region-reflective';'interior-point';'trust-region-dogleg';'trust-region'

% Set up options for the optimization step
if strcmp(optim_fcn,'fminbnd')
    optOptim = optimset('Display','off','TolX',1.5e-8,'MaxIter',100); % set control parameters
elseif strcmp(optim_fcn,'fmincon') && strcmp(Algorithm,'sqp')
    optOptim = optimoptions('fmincon','Algorithm','sqp','Display','off',...
        'StepTolerance',1.5e-8,'OptimalityTolerance',1.5e-8,...
        'SpecifyObjectiveGradient',true,'CheckGradients',false,'HessianFcn',[]);
elseif strcmp(optim_fcn,'fmincon') && strcmp(Algorithm,'trust-region-reflective')
    optOptim = optimoptions('fmincon','Algorithm',Algorithm,'Display','off',...
        'StepTolerance',1.5e-8,'OptimalityTolerance',1.5e-8,'FunctionTolerance',1e-6,...
        'SpecifyObjectiveGradient',true,'CheckGradients',false,'HessianFcn','objective');
elseif strcmp(optim_fcn,'fmincon') && strcmp(Algorithm,'interior-point')
    optOptim = optimoptions('fmincon','Algorithm','interior-point','Display','off',...
        'StepTolerance',1.5e-8,'OptimalityTolerance',1.5e-8,...
        'SpecifyObjectiveGradient',true,'CheckGradients',false,'HessianApproximation','bfgs');
elseif strcmp(optim_fcn,'fzero')
    % change initial value and policy function to avoid corner solutions for the 1st iteration
    dec(:) = zeros(nTotalGrid,1); % the initial capital choice is zero
    v(:) = bellmanMax_vec(dec(:),total_vec,sigma,beta,prob_mat,zeros(nTotalGrid,nShock));
    optOptim = optimset('Display','off','TolX',2.2e-16); % set control parameters
elseif strcmp(optim_fcn,'fsolve') && strcmp(Algorithm,'trust-region-dogleg')
    % change initial value and policy function to avoid corner solutions for the 1st iteration
    dec(:) = zeros(nTotalGrid,1); % the initial capital choice is zero
    v(:) = bellmanMax_vec(dec(:),total_vec,sigma,beta,prob_mat,zeros(nTotalGrid,nShock));
    optOptim = optimoptions('fsolve','Algorithm','trust-region-dogleg','Display','off',...
        'StepTolerance',1.5e-8,'OptimalityTolerance',1.5e-8,...
        'SpecifyObjectiveGradient',true,'CheckGradients',false); 
elseif strcmp(optim_fcn,'fsolve') && strcmp(Algorithm,'trust-region')
    % change initial value and policy function to avoid corner solutions for the 1st iteration
    dec(:) = zeros(nTotalGrid,1); % the initial capital choice is zero
    v(:) = bellmanMax_vec(dec(:),total_vec,sigma,beta,prob_mat,zeros(nTotalGrid,nShock));    
    optOptim = optimoptions('fsolve','Algorithm','trust-region','Display','off',...
        'StepTolerance',1.5e-8,'OptimalityTolerance',1.5e-8,...
        'SpecifyObjectiveGradient',true,'CheckGradients',false);
else
    optOptim = optimset('Display','off','TolX',1.5e-8,'MaxIter',100); % set control parameters
end

% output values of fmin
xmin_vec = NaN(nTotalGrid,1);   % optimizers
fmin_vec = NaN(nTotalGrid,1); % function values for minimization
gmin_vec = NaN(nTotalGrid,1); % function values for derivative of Bellman equation
jmin_vec = NaN(nTotalGrid,1); % function values for Jacobian of Bellman equation
exitflag_vec = NaN(nTotalGrid,1); % exitflag
output_vec = cell(nTotalGrid,1); % output parameters as a structure
iterOptim_vec = NaN(nTotalGrid,1); % number of iterations
funcCountOptim_vec = NaN(nTotalGrid,1); % number of function evaluations

%
% 2.2 Value function iteration
%
fprintf(1,'%12s%12s%12s\n','niter_v','metric_v','metric_p'); % Print out Iteration Header

for niter_v = 1:maxiter_v
    % interpolate continuation value function
    vfn = spline(agrid,v.');   % interpolate value function using cubic splines with not-a-knot condition
    vfnDer = fnder(vfn,1);     % the derivative of spline
    vfnHess = fnder(vfn,2);    % the Hessian of spline
    
%     vfn = mypolyfit1(agrid,v.',polyOrder,polyInterval,polyform); % interpolate value function using monomial polynomial
%     vfnDer = vfn; % for polynomial, we take derivative in the original representation
%     vfnHess = vfn; % for polynomial, we take derivative in the original representation

    % Solve Bellman Equation Optimization Problem
    if strcmp(optim_fcn,'fminbnd')
        for ii = 1:nTotalGrid
            fmin_objFcn = @(xx) bellmanMin(xx,total_vec(ii),sigma,beta,prob_mat(ii,:),vfn);
            [xmin_vec(ii),fmin_vec(ii),exitflag_vec(ii),output_vec{ii}] = fminbnd(fmin_objFcn,lb_vec(ii),ub_vec(ii),optOptim);
            iterOptim_vec(ii) = output_vec{ii}.iterations;
            funcCountOptim_vec(ii) = output_vec{ii}.funcCount;            
        end
        tv = - fmin_vec; % convert back to maximization problem
        tdec = xmin_vec; % the policy function
    elseif strcmp(optim_fcn,'fmincon') && strcmp(optOptim.Algorithm,'sqp')
        for ii = 1:nTotalGrid
            fmin_objFcn = @(xx) bellmanMin(xx,total_vec(ii),sigma,beta,prob_mat(ii,:),vfn,vfnDer);
            [xmin_vec(ii),fmin_vec(ii),exitflag_vec(ii),output_vec{ii}] = fmincon(fmin_objFcn,x0_vec(ii),[],[],[],[],...
                                                                                  lb_vec(ii),ub_vec(ii),[],optOptim);
            iterOptim_vec(ii) = output_vec{ii}.iterations;
            funcCountOptim_vec(ii) = output_vec{ii}.funcCount;            
        end
        tv = - fmin_vec; % convert back to maximization problem
        tdec = xmin_vec; % the policy function
    elseif strcmp(optim_fcn,'fmincon') && strcmp(optOptim.Algorithm,'trust-region-reflective')
        for ii = 1:nTotalGrid
            fmin_objFcn = @(xx) bellmanMin(xx,total_vec(ii),sigma,beta,prob_mat(ii,:),vfn,vfnDer,vfnHess);
            [xmin_vec(ii),fmin_vec(ii),exitflag_vec(ii),output_vec{ii}] = fmincon(fmin_objFcn,x0_vec(ii),[],[],[],[],...
                                                                                  lb_vec(ii),ub_vec(ii),[],optOptim);
            iterOptim_vec(ii) = output_vec{ii}.iterations;
            funcCountOptim_vec(ii) = output_vec{ii}.funcCount;            
        end
        tv = - fmin_vec; % convert back to maximization problem
        tdec = xmin_vec; % the policy function
    elseif strcmp(optim_fcn,'fmincon') && strcmp(optOptim.Algorithm,'interior-point')
        for ii = 1:nTotalGrid
            fmin_objFcn = @(xx) bellmanMin(xx,total_vec(ii),sigma,beta,prob_mat(ii,:),vfn,vfnDer);
            [xmin_vec(ii),fmin_vec(ii),exitflag_vec(ii),output_vec{ii}] = fmincon(fmin_objFcn,x0_vec(ii),[],[],[],[],...
                lb_vec(ii),ub_vec(ii),[],optOptim);
        end
        iterOptim_vec(ii) = output_vec{ii}.iterations;
        funcCountOptim_vec(ii) = output_vec{ii}.funcCount;
        tv = - fmin_vec; % convert back to maximization problem
        tdec = xmin_vec; % the policy function
    elseif strcmp(optim_fcn,'fzero')
        for ii = 1:nTotalGrid
            gzero_objFcn = @(xx) bellmanMinFOC(xx,total_vec(ii),sigma,beta,prob_mat(ii,:),vfnDer);
            [xmin_vec(ii),gmin_vec(ii),exitflag_vec(ii),output_vec{ii}] = fzero(gzero_objFcn,[lb_vec(ii),ub_vec(ii)],optOptim);
            iterOptim_vec(ii) = output_vec{ii}.iterations;
            funcCountOptim_vec(ii) = output_vec{ii}.funcCount;
            % compute Bellman value at the final solution: we also compute Jacobian to check SOC
            [fmin_vec(ii),gmin_vec(ii),jmin_vec(ii)] = bellmanMin(xmin_vec(ii),total_vec(ii),sigma,beta,prob_mat(ii,:),vfn,vfnDer,vfnHess);
        end
        tv = - fmin_vec; % convert back to maximization problem
        tdec = xmin_vec; % the policy function
    elseif strcmp(optim_fcn,'fsolve')
        for ii = 1:nTotalGrid
            gzero_objFcn = @(xx) bellmanMinFOC(xx,total_vec(ii),sigma,beta,prob_mat(ii,:),vfnDer,vfnHess);
            [xmin_vec(ii),gmin_vec(ii),exitflag_vec(ii),output_vec{ii},jmin_vec(ii)] = fsolve(gzero_objFcn,x0_vec(ii),optOptim);
            iterOptim_vec(ii) = output_vec{ii}.iterations;
            funcCountOptim_vec(ii) = output_vec{ii}.funcCount;
            % compute Bellman value at the final solution
            fmin_vec(ii) = bellmanMin(xmin_vec(ii),total_vec(ii),sigma,beta,prob_mat(ii,:),vfn);
        end
        tv = - fmin_vec; % convert back to maximization problem
        tdec = xmin_vec; % the policy function
    else
         % Maximize the Bellman equation
%          [tdec,tv,exitflag] = goldenx(@bellmanMax_vec,lb_vec,ub_vec,optOptim.MaxIter,optOptim.TolX,total_vec,sigma,beta,prob_mat,vfn);
        error('VFI: The Optimization Algorithm is not Defined');
    end
        
    % calculating the change of value and policy function
    metric_v = max(abs(tv - v(:))./(abs(v(:)) + 1.0));
    metric_p = max(abs(tdec - dec(:))./(abs(dec(:)) + 1.0));
    
    fprintf(1,'%12d%12.4g%12.4g\n',[round(niter_v),metric_v,metric_p]); % print out the iteration results
    % Check the convergence
    if (metric_v <= tol_v) && (metric_p <= tol_p) % converged: break out of the loop
        break;
    elseif (metric_v > tol_v) && (metric_p <= tol_hp) && (niter_v >= 2) % policy converged (excluding 1st iter): Howard Improvement
        dec(:) = tdec;        % update policy function
        v(:) = tv;            % update value function
        x0_vec = tdec;        % update initial value
        
        % (Modified) Howard Improvement Step
        for niter_h = 1:maxiter_h
            vfn = spline(agrid,v.');   % interpolate value function using cubic splines with not-a-knot condition
%             vfn = mypolyfit1(agrid,v.',polyOrder,polyInterval,polyform); % interpolate value function using monomial polynomial
            tv_h = bellmanMax_vec(dec(:),total_vec,sigma,beta,prob_mat,vfn);
            metric_h = max(abs(tv_h - v(:))./(abs(v(:)) + 1.0));
            if metric_h <= tol_hv
                break;
            else
                v(:) = tv_h;
            end
        end
    else % stick to safe method
        dec(:) = tdec;        % update policy function
        v(:) = tv;            % update value function
        x0_vec = tdec;        % update initial value        
    end
end

%
% Game over: how fast is my job?
%
time_spent = toc(time_start); % record time    
fprintf(1,'Time Spent in Computation: %14.4g Seconds\n',time_spent);

% plot the result
figure(1)
plot(agrid,v);
title('Value Function');
xlabel('Asset');
ylabel('Value Function');

figure(2)
plot(agrid,dec);
title('Policy Function');
xlabel('Asset');
ylabel('Policy Function');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local Functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local Function: Scalar Bellman equation function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [bv,bvDer,bvHess] = bellmanMin(aprime,total,sigma,beta,prob,vfn,vfnDer,vfnHess)

% Consumption
cons = total - aprime;

% Bellman Equation Value

% Corner Cases: Invalid Consumption
if cons < eps % Zero or Negative Consumption: invalid case
    % give -Inf to the original Bellman
    bv = -realmax;
    bvDer = -realmax;
    bvHess = -realmax;
    % return the negative value of Bellman as we are solving a min problem
    bv = - bv;
    bvDer = - bvDer;
    bvHess = - bvHess;
    % exit
    return;
end

% Valid cases: we can safely assume that consumption is positive
% Flow utility
if abs(sigma - 1.0) < eps % log utility
    util = log(cons);
else % non-log CRRA utility
    util = (cons.^(1-sigma))./(1-sigma);
end
% Continuation Value
if isa(vfn,'struct') && strcmp(vfn.form,'pp') % a structure representing splines
    contvalue = ppval(vfn,aprime);
elseif isa(vfn,'struct') && strncmpi(vfn.form,'poly',4) % a structure representing splines
    contvalue = mypolyval1(vfn,aprime);
end
% Bellman value
bv = util + beta*prob*contvalue;

% Derivative
bvDer = NaN;
if nargout > 1
    utilDer = - cons.^(-sigma);
    if isa(vfn,'struct') && strcmp(vfn.form,'pp') % a structure representing splines
        contvalueDer = ppval(vfnDer,aprime);
    elseif isa(vfn,'struct') && strncmpi(vfn.form,'poly',4) % a structure representing splines
        error('Derivative for polynomial is not currently supported');
        %     contvalueDer = mypolyval1(vfn,aprime);
    end
    bvDer = utilDer + beta*prob*contvalueDer;
end

% Hessian
bvHess = NaN;
if nargout > 2
    utilHess = utilDer*sigma/cons;
    if isa(vfn,'struct') && strcmp(vfn.form,'pp') % a structure representing splines
        contvalueHess = ppval(vfnHess,aprime);
    elseif isa(vfn,'struct') && strncmpi(vfn.form,'poly',4) % a structure representing splines
         error('Derivative for polynomial is not currently supported');
%         contvalueHess = mypolyval1(vfn,aprime);
    end    
    bvHess = utilHess + beta*prob*contvalueHess;
end

% return the negative value of Bellman as we are solving a min problem
bv = - bv;
bvDer = - bvDer;
bvHess = - bvHess;

end % end the local function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local Function: Scalar Bellman equation FOC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [bvDer,bvHess] = bellmanMinFOC(aprime,total,sigma,beta,prob,vfnDer,vfnHess)

% Consumption
cons = total - aprime;

if cons < eps % Zero or Negative Consumption: invalid case
    % give -Inf to the original Bellman
    bvDer = - realmax;
    bvHess = - realmax;
    % return the negative value of Bellman as we are solving a min problem
    bvDer = - bvDer;
    bvHess = - bvHess;
    % exit
    return;
end

% Valid cases: we can safely assume that consumption is positive
% Derivative
utilDer = - cons.^(-sigma);
if isa(vfnDer,'struct') && strcmp(vfnDer.form,'pp') % a structure representing splines
    contvalueDer = ppval(vfnDer,aprime);
elseif isa(vfnDer,'struct') && strncmpi(vfnDer.form,'poly',4) % a structure representing splines
    error('Derivative for polynomial is not currently supported');
%     contvalueDer = mypolyval1(vfn,aprime);
end

bvDer = utilDer + beta*prob*contvalueDer;

% Hessian
bvHess = NaN;
if nargout > 1
    utilHess = utilDer*sigma/cons;
    if isa(vfnHess,'struct') && strcmp(vfnHess.form,'pp') % a structure representing splines
        contvalueHess = ppval(vfnHess,aprime);
    elseif isa(vfnHess,'struct') && strncmpi(vfnHess.form,'poly',4) % a structure representing splines
        error('Derivative for polynomial is not currently supported');
%         contvalueHess = mypolyval1(vfn,aprime);
    end    
    bvHess = utilHess + beta*prob*contvalueHess;
end

% return the negative value of Bellman as we are solving a min problem
bvDer = - bvDer;
bvHess = - bvHess;

end % end the local function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Local Function: Vector Bellman equation function for max problem
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function tv_vec = bellmanMax_vec(aprime_vec,total_vec,sigma,beta,prob_mat,vfn)

% Consumption
cons_vec = total_vec - aprime_vec;

% Index for positive consumption
isvalid_vec = find(cons_vec > 0);

% Flow value for CRRA
util_vec = - repmat(realmax,size(cons_vec)); % give default flow utility minus infinity
if abs(sigma - 1.0) < eps % log utility
    util_vec(isvalid_vec) = log(cons_vec(isvalid_vec)); % update the util 
else % non-log CRRA utility
    util_vec(isvalid_vec) = ((cons_vec(isvalid_vec)).^(1-sigma))./(1-sigma); % update the util 
end

% Continuation Value
if isa(vfn,'numeric') % a continuation value matrix
    contvalue_vec = vfn;
elseif isa(vfn,'struct') && strcmp(vfn.form,'pp') % a structure representing splines
    contvalue_vec = (ppval(vfn,aprime_vec.')).';
elseif isa(vfn,'struct') && strncmpi(vfn.form,'poly',4) % a structure representing splines
    contvalue_vec = (mypolyval1(vfn,aprime_vec.')).';
end

% Bellman value
tv_vec = util_vec + beta*sum(prob_mat.*contvalue_vec,2);

end % end the local function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Subfunction: chebynodes1
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nodes_unit,nodes_interval] = chebynodes1(n,interval)

% chebynodes: generate 1-D Chebyshev nodes on an interval
% Author: Jinhui Bai (jinhui.bai@gmail.com), 05/21/2020

fractionMin = pi*0.5/n; % min fraction
dec = - 2*fractionMin; % decrement
fractionMax = pi - fractionMin; % max fraction
fraction = (fractionMax:dec:fractionMin).'; % trigonometric fraction on [0,pi]
nodes_unit = cos(fraction); % Cheby nodes on [-1, 1]
if nargout > 1
    left = interval(1);
    right = interval(2);
    nodes_interval = (left + right)/2 + nodes_unit*(right - left)/2; % Cheby nodes on [left,right]
end

end % end subfunction chebynodes1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Subfunction: mypolyfit1
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function poly = mypolyfit1(breaks,Values,order,interval,polyform)

% mypolyfit1 constructs a vector-valued 1-D Chebyshev or Monomial polynomial
% Author: Jinhui Bai (jinhui.bai@gmail.com), 05/21/2020

if nargin < 5, polyform = 'polycheby';end
if nargin < 4, interval = [-1,1];end
if nargin < 3, order = size(Values,2);end

% normalize breaks to the interval [-1,1]
n = size(Values,2); % # of data points in curve fitting
if isempty(breaks) % no nodes provided, use Chebyshev nodes
    nodes_unit = chebynodes1(n);
else % nodes provided, may or may not be Chebyshev nodes
    nodes = reshape(breaks,numel(breaks),1);
    left = interval(1);
    right = interval(2);
    nodes_unit = (2*nodes - left - right)/(right - left);
end

% Generate interpolation matrix 
% Coef Order: use MATLAB polynomial representation to have higher order term appear first, the opposite to a typical textbook order
if strncmpi(polyform,'polycheby',8) % Chebyshev Polynomial
    interpMat(:,[order-1,order]) = [nodes_unit,ones(n,1)]; % initialize the interpolation matrix
    for j = (order-2):-1:1
        interpMat(:,j) = 2*nodes_unit.*interpMat(:,j+1) - interpMat(:,j+2);
    end
elseif strncmpi(polyform,'polymonomial',8) % Monomial Polynomial
    interpMat(:,order) = ones(n,1); % initialize the interpolation matrix
    for j = (order-1):-1:1
        interpMat(:,j) = nodes_unit.*interpMat(:,j+1);
    end    
end

% polynomial coefficients
coefs = interpMat\(Values.'); % solve the linear equation
% coefs_analytical = (interpMat.')*(Values.')*(2/n);
% coefs_analytical(order,:) = coefs_analytical(order,:)/2;
% diff_coefs = coefs_analytical - coefs
coefs = coefs.'; % transpose back to Curve Fitting Toolbox order

% construct a polynomial structure
d = size(Values,1); % dimension of curve fitting
poly = struct('form',polyform,'breaks',breaks,'coefs',coefs,'Values',Values,'interval',interval,'order',order,'dim',d);

end % end subfunction mypolyfit1

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Subfunction: mypolyval1
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function polyval = mypolyval1(poly,x)

% mypolyval1 evalutes a vector-valued polynomial at inquiry vector x
% Author: Jinhui Bai (jinhui.bai@gmail.com), 05/21/2020

% normalize evaluation points to the interval [-1,1]
left = poly.interval(1);
right = poly.interval(2);
x_unit = (2*x(:) - left - right)/(right - left);

% Evaluate the polynomial
polyform = poly.form;
coefs = poly.coefs;
dim = poly.dim;
order = poly.order;
n = length(x_unit);
if strncmpi(polyform,'polycheby',8) % Chebyshev Polynomial
    interpMat(:,[order-1,order]) = [x_unit,ones(n,1)]; % initialize the interpolation matrix
    for j = (order-2):-1:1
        interpMat(:,j) = 2*x_unit.*interpMat(:,j+1) - interpMat(:,j+2);
    end
    polyval = coefs*(interpMat.'); % the value of polynomial
elseif strncmpi(polyform,'polymonomial',8) % Monomial Polynomial
    % use nested multiplication (See Numerical Recipe, 3rd Edition, Chapter 5.1)
    x_eval = reshape(repmat(x_unit.',[dim 1]),[dim*n,1]);
    coefs_eval = repmat(coefs,[n 1]);
    polyval = coefs_eval(:,1);
    for j = 2:order
        polyval = polyval.*x_eval + coefs_eval(:,j);
    end
    polyval = reshape(polyval,[dim,n]); % the value of polynomial
end

end % end subfunction mypolyval1

