function [x,fval,exitflag,output] = myfsolve(fun,x0,options)

% Author: Jinhui Bai (jinhui.bai@gmail.com), Washingtong State University
% Version History:
% 0.2 January, 2016
% 0.1 September, 2015
% 0.0 July, 2015

% set up default options
defaultopt = ...
    struct(...
    'Algorithm','Broyden',...
    'Display','iter',...
    'FinDiffRelStep',sqrt(eps),...
    'FinDiffType','forward',...                           
    'InitStepSize',1.0,...
    'Jacobian','off',...
    'Jacobian0',[],...    
    'JacobianInv0',[],...
    'LineSearchMethod','Parabolic3',...
    'MaxFunEvals',400,...
    'MaxIter',100,...
    'MaxLineSearch',10,...    
    'NormOrder',Inf,...
    'Restarts',true,...
    'ScaleProblem','None',...    
    'TolFun',1e-8,...
    'TolX',eps,...
    'TypicalX',[]...
    );

% get and check the options
algorithm = optimget(options,'Algorithm',defaultopt,'fast');
Display = optimget(options,'Display',defaultopt,'fast');
FinDiffRelStep   = optimget(options,'FinDiffRelStep',defaultopt,'fast');
initStepSize   = optimget(options,'InitStepSize',defaultopt,'fast');
Jacobian = optimget(options,'Jacobian',defaultopt,'fast');
Jacobian0 = optimget(options,'Jacobian0',defaultopt,'fast');
JacobianInv0 = optimget(options,'JacobianInv0',defaultopt,'fast');
lineSearchMethod   = optimget(options,'LineSearchMethod',defaultopt,'fast');
maxFunEvals   = optimget(options,'MaxFunEvals',defaultopt,'fast');
maxIter   = optimget(options,'MaxIter',defaultopt,'fast');
maxLineSearch   = optimget(options,'MaxLineSearch',defaultopt,'fast');
normOrder   = optimget(options,'NormOrder',defaultopt,'fast');
restarts   = optimget(options,'Restarts',defaultopt,'fast');
scaleProblem   = optimget(options,'ScaleProblem',defaultopt,'fast');
tolFun   = optimget(options,'TolFun',defaultopt,'fast');
tolX   = optimget(options,'TolX',defaultopt,'fast');
typicalX   = optimget(options,'TypicalX',defaultopt,'fast');

% fvalBounds = [-1.0;-0.01];
% determine the algorithm type
isFixedJacobian = false;
isBroyden = false;
isBroydenInv = false;
isCauchy = false;
isChord = false;
isNewton = false;
if strcmp(algorithm,'FixedJacobian')
    isFixedJacobian = true;
elseif strcmp(algorithm,'Broyden')
    isBroyden = true;
elseif strcmp(algorithm,'BroydenInv')
    isBroydenInv = true;
elseif strcmp(algorithm,'Cauchy')
    isCauchy = true;        
elseif strcmp(algorithm,'Chord')
    isChord = true;
elseif strcmp(algorithm,'Newton')
    isNewton = true;
end

% determine the line search method
isArmijo = false;
isParabolic3 = false;
if strcmp(lineSearchMethod,'Armijo')
    isArmijo = true;
elseif strcmp(lineSearchMethod,'Parabolic3')
    isParabolic3 = true;    
end

% calculate the initial function value and Jacobian (if needed)
nx = length(x0(:));
if ~isempty(Jacobian0) && isscalar(Jacobian0) % change it to a multiple of identity matrix
    Jacobian0 = Jacobian0*eye(nx,nx);
end
if ~isempty(JacobianInv0) && isscalar(JacobianInv0) % change it to a multiple of identity matrix
    JacobianInv0 = JacobianInv0*eye(nx,nx);
end

fjac0 = Jacobian0;
fjacinv0 = JacobianInv0;
needJacobianToRestart = false;
JacobianRestarted = false;
isJacobian = false;
if strcmp(Jacobian,'on'),isJacobian = true;end

if isFixedJacobian
    f0 = fun(x0);
    funcCount = 1;
    if isempty(fjacinv0) && isempty(fjac0)
        error('Either Jacobian0 or JacobianInv0 should be nonempty');
    elseif isempty(fjacinv0) && ~isempty(fjac0)
        fjacinv0 = inv(fjac0);
    end
    JacobianRestarted = false;
elseif isCauchy || isBroyden || isNewton
    if ~isempty(fjac0) || (isempty(fjac0) && ~isJacobian) % evaluate function value only
        f0 = fun(x0);
    elseif isempty(fjac0) && ~isJacobian % evaluate function value only (Jacobian is computed later inside the loop)
        f0 = fun(x0);
    elseif isempty(fjac0) && isJacobian % evaluate function and Jacobian at the same time
        [f0,fjac0] = fun(x0);
        JacobianRestarted = true; % true Jacobian is calculated
    end
    funcCount = 1;
elseif isBroydenInv || isChord
    if ~isempty(fjacinv0) % evaluate function value only
        f0 = fun(x0);
    elseif isempty(fjacinv0) && ~isJacobian % evaluate function value only (Jacobian is computed later inside the loop)
        f0 = fun(x0);
    elseif isempty(fjacinv0) && isJacobian  % evaluate function and Jacobian at the same time
        [f0,fjac0] = fun(x0);
        fjacinv0 = inv(fjac0);
        JacobianRestarted = true; % true Jacobian is calculated
    end
    funcCount = 1;    
end
fnorm_0 = norm(f0,normOrder); % the specified norm
fnorm2_0 = norm(f0,2);  % 2-norm used in line search

%
% calculate the initial scale vector
%

scalemin = eps;
scalemax = 1/scalemin;
%
if strcmp(scaleProblem,'TypicalX')
    if isempty(typicalX) % default value is x0
        xScale = abs(x0(:));        
    else
        xScale = abs(typicalX);        
    end
    xScale = max(min(scalemax,xScale),scalemin);
elseif strcmp(scaleProblem,'Jacobian') % a placeholder for now
    xScale = ones(nx,1);
%     xScale = getxScale(nx,JAC,scalemin,scalemax);
else
    xScale = ones(nx,1);
end
% xScaleDiagMat = diag(xScale);
% xScaleDiagMatInv = diag(1./xScale);

fvalScale = ones(nx,1); % place holder for normalization

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Start the solution iteration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
x = x0(:)./xScale; % rescaled x0
fval = f0(:)./fvalScale;  % keep fval scale as it is
if ~isempty(fjac0) % multiply for rescaled Jacobian
    % equivalent to: fjac0*diag(xScale)
    fjac = bsxfun(@times,fjac0,xScale(:).');
    fjac_rank = rank(fjac);
else % rescale later after we calculate fjac
    fjac = fjac0;
end 
if ~isempty(fjacinv0) % multiply for rescaled Inverse Jacobian
    % equivalent to: diag(1./xScale)*fjac0
    fjacinv = bsxfun(@times,1.0./xScale(:),fjacinv0);
else
    fjacinv = fjacinv0;
end 
fnorm = fnorm_0; % fnorm in the original scale
fnorm2 = fnorm2_0; % 2-norm of f in the original scale
tolfnorm = tolFun; % unscaled tolerance
% tolfnorm = tolFun*(1 + fnorm_0); % scaled tolerance by the initial norm (See Kelly, 1995) 
Armijo_low = 1e-4; % Armijo sufficient descent parameter
% Armijo_high = 0.9999;
safeGuard_low = 0.1;
safeGuard_high = 0.5;

% print out the initial statistics
if strncmp(Display,'iter',4) || strncmp(Display,'final',5)
    fprintf('\n%14s %14s %14s %14s %14s %14s %14s %14s\n',...
        'Iteration','Func-count','Line Search','fnorm(x)   ','fnorm2(x)   ','Norm of Step','Step Size','Algorithm');
    fprintf('\n%14i %14i %14i %14.6e %14.6e\n',[0 funcCount 0 fnorm fnorm2]); %f0
end

for iter = 1:maxIter
    if fnorm <= tolfnorm
        exitflag = 1;
        [x,fval,output] = outputfcn(x,fval,iter - 1,funcCount,algorithm,fjac,fjacinv,xScale,fvalScale);
        return;
    elseif funcCount >= maxFunEvals
        exitflag = 0;
        [x,fval,output] = outputfcn(x,fval,iter - 1,funcCount,algorithm,fjac,fjacinv,xScale,fvalScale);
        return;
    elseif isequal(iter,1) % for the first iteration, calculate Numerical Jacobian if not computed before
        stepAlgorithm = algorithm; % Algorithm used in this step        
        if (isCauchy || isBroyden || isNewton) && (isempty(fjac0) && ~isJacobian)
            fjac0 = fdjac1(fun,x0,f0,FinDiffRelStep);
            funcCount = funcCount + nx;
            fjac = bsxfun(@times,fjac0,xScale(:).'); % rescale the Jacobian
            fjac_rank = rank(fjac); % check the rank deficiency of the Jacobian
            JacobianRestarted = true; % true Jacobian is calculated
            if isBroyden
                stepAlgorithm = 'Newton'; % Algorithm used in this step
            end
        elseif (isBroydenInv || isChord) && (isempty(fjacinv0) && ~isJacobian)
            fjac0 = fdjac1(fun,x0,f0,FinDiffRelStep);
            funcCount = funcCount + nx;
            fjacinv0 = inv(fjac0);            
            fjacinv = bsxfun(@times,1.0./xScale(:),fjacinv0); % rescale the inverse Jacobian 
            JacobianRestarted = true; % true Jacobian is calculated
            stepAlgorithm = 'Newton'; % Algorithm used in this step
        end
    elseif iter > 1 % update the Jacobian starting from the second iteration
        if isBroyden && ~needJacobianToRestart && all(dx >= dxThreshold) % update Jacobian: Broyden's update formula
            fjac = fjac + (df-fjac*dx)*dx'/(dx'*dx);
            fjac_rank = rank(fjac);%fjac_rank % check the rank deficiency of the Jacobian: used in the update step
            JacobianRestarted = false;% fjac
            stepAlgorithm = 'Broyden'; % Algorithm used in this step
        elseif isBroyden && ~needJacobianToRestart && ~all(dx >= dxThreshold) % no update for Jacobian: Fixed Jacobian
            %fjac_rank = rank(fjac);%fjac_rank
            JacobianRestarted = false;% fjac
            stepAlgorithm = 'FixedJacobian'; % Algorithm used in this step            
        elseif isBroydenInv && ~needJacobianToRestart && all(dx >= dxThreshold) % update Inverse Jacobian: Broyden's inverse update formula
            temp = fjacinv*df;
            fjacinv = fjacinv + (dx-temp)*(dx'*fjacinv)/(dx'*temp);
            JacobianRestarted = false;
            stepAlgorithm = 'BroydenInv'; % Algorithm used in this step
        elseif isBroydenInv && ~needJacobianToRestart && ~all(dx >= dxThreshold) % no update for Jacobian: Fixed Jacobian
            JacobianRestarted = false;% fjac
            stepAlgorithm = 'FixedJacobian'; % Algorithm used in this step                        
        elseif isCauchy || isNewton || ((isBroyden || isBroydenInv || isChord) && needJacobianToRestart) % calculate Jacobian directly
            if isJacobian
                [fvalTemp,fjacTemp] = fun(x.*xScale);
                fval = fvalTemp./fvalScale;
                funcCount = funcCount + 1;
                JacobianRestarted = true;
            else
                fjacTemp = fdjac1(fun,x.*xScale,fval,FinDiffRelStep); % calculate the Jacobian in the original space
                funcCount = funcCount + nx;
                JacobianRestarted = true;
            end
            fjac = bsxfun(@times,fjacTemp,xScale(:).');%fjac % the Jacobian in the normalized space
            fjac_rank = rank(fjac);%fjac_rank % check the rank deficiency of the Jacobian
            if isCauchy
               stepAlgorithm = 'Cauchy'; % Algorithm used in this step
            elseif isBroydenInv || isChord % the Inverse Jacobian in the normalized space                
                if isequal(fjac_rank,size(fjac,1))
                    fjacinv = inv(fjac);
                    stepAlgorithm = 'Newton'; % Algorithm used in this step
                else % if rank deficient, switch to Cauchy step, i.e., fjacinv = fjac.'
%                     fjacinv = fjacinv0;
                    fjacinv = fjac.';
                    stepAlgorithm = 'Cauchy'; % Algorithm used in this step
                end
            elseif isBroyden || isNewton
                if isequal(fjac_rank,size(fjac,1))
                    stepAlgorithm = 'Newton'; % Algorithm used in this step
                else % if rank deficient, switch to Cauchy step, i.e., fjacinv = fjac.'
                    stepAlgorithm = 'Cauchy'; % Algorithm used in this step
                end                 
            end            
        elseif isChord && ~needJacobianToRestart % Chord Method: No need for update
            %             fjac = fjac;
            JacobianRestarted = false;
            stepAlgorithm = 'Chord'; % Algorithm used in this step
        elseif isFixedJacobian % fix the Jacobian: No need for update
%             fjacinv = fjacinv;
            JacobianRestarted = false;
            stepAlgorithm = 'FixedJacobian'; % Algorithm used in this step
        end        
    end
    % calculate the full step direction for Newton or Quasi-Newton Method in the normalized space
    if isCauchy || ((isBroyden || isNewton) && ~isequal(fjac_rank,size(fjac,1))) % Cauchy direction for 1/2 * fval'*fval
        dxFullStep = -fjac.'*fval;
    elseif (isBroyden || isNewton) && isequal(fjac_rank,size(fjac,1))
        dxFullStep = -fjac\fval;
    elseif isFixedJacobian || isBroydenInv || isChord
        dxFullStep = -fjacinv*fval;
        %fval
    end
%     dxFullStepNorm = norm(dxFullStep,normOrder);
    % backtracking line search based on the full step
    lambdaTrial = initStepSize; % backtracking parameter
    for iterLineSearch = 1:maxLineSearch
        dxTrial = lambdaTrial*dxFullStep; % new dx trial in the normalized space
        xTrial = x + dxTrial; % new x trial in the normalized space
        fvalTrialTemp = fun(xTrial.*xScale); % evaluate function in the original (x,fval) space        
        fnormTrial = norm(fvalTrialTemp,normOrder); % Important: fnorm in the original fval space
%         dxnormTrial = norm(dxTrial.*xScale,normOrder);  % Important: dxnorm in the original x space
        fnorm2Trial = norm(fvalTrialTemp,2); % Important: fnorm in the original fval space
        dxnorm2Trial = norm(dxTrial.*xScale,2);  % Important: dxnorm in the original x space        
        funcCount = funcCount + 1;
        fvalTrial = fvalTrialTemp./fvalScale; % fval in the normalized space
        
        if strncmp(Display,'iter',4)
            fprintf('\n%14i %14i %14i %14.6e %14.6e %14.6e %14.6e %14s\n',...
                iter,funcCount,iterLineSearch,fnormTrial,fnorm2Trial,dxnorm2Trial,lambdaTrial,stepAlgorithm);
%             x
%             dxTrial
%             xTrial            
%             fvalTrialTemp
        end
        % test decent from the function value form of Armijo rule (Kelly, 1995)
        test_Armijo = (fnorm2Trial < (1 - Armijo_low*lambdaTrial)*fnorm2);
%         test_fvalBounds = all(fvalTrialTemp > (fvalBounds(:,1) + 1e-16));test_fvalBounds
        if test_Armijo
            break;
        elseif isArmijo && (iterLineSearch < maxLineSearch) 
            lambdaTrial = safeGuard_high*lambdaTrial;
        elseif isParabolic3 && (iterLineSearch < maxLineSearch)            
            if isequal(iterLineSearch,1) % update based on Armijo's rule and move to the second iteration                
                lambdaTrialNew = safeGuard_high*lambdaTrial; % update                
            elseif iterLineSearch > 1 % update based on the 3-point parabolic rule
                lambdaTrialNew = parab3p(lambdaTrial,lambdaTrial_last,fnorm,fnorm2Trial,fnorm2Trial_last,safeGuard_low,safeGuard_high);
            end
            fnorm2Trial_last = fnorm2Trial;
            lambdaTrial_last = lambdaTrial; % record the last lambda
            lambdaTrial = lambdaTrialNew; % udpate lambda trial            
        end        
    end
    %
    % check convergence and update
    %
    if (~test_Armijo)  % failed line search
        if isCauchy || isNewton || ((isBroyden || isBroydenInv || isChord) && JacobianRestarted) % Newton: exit; Chord or Quasi-Newton: exit if just restarted
            exitflag = -4;
            [x,fval,output] = outputfcn(x,fval,iter,funcCount,algorithm,fjac,fjacinv,xScale,fvalScale);
            return;
        elseif (isBroyden || isBroydenInv || isChord)  && ~JacobianRestarted && restarts
            needJacobianToRestart = true;
            % keep the old value: no update to x, fval and fnorm
            % x = x;
            % fval = fval;
            % fnorm = fnorm;
            % fnorm2 = fnorm2;
        elseif isFixedJacobian % no need for restart: move on to the last evaluated point
            needJacobianToRestart = false;
            % update to the new value
            x = xTrial;
            fval = fvalTrial;
            fnorm = norm(fval,normOrder);
            fnorm2 = fnorm2Trial;
        else % no need for restart: move on to the last evaluated point
            needJacobianToRestart = false;
            % calculate the change
            dx = dxTrial;
            df = fvalTrial - fval;
            % safeguard threshold to avoid too small steps: no more than the numerical derivative step
            dxThreshold = max(FinDiffRelStep.*abs(x),eps); 
            % update to the new value
            x = xTrial;
            fval = fvalTrial;
            fnorm = norm(fval,normOrder);
            fnorm2 = fnorm2Trial;
        end
        % fjacinv = eye(size(x,1));
    else % successful line search
        if strncmp(Display,'final',5)
            fprintf('\n%14i %14i %14i %14.6e %14.6e %14.6e %14.6e\n',...
                [iter funcCount iterLineSearch fnormTrial fnorm2Trial dxnorm2Trial lambdaTrial]);
        end        
        needJacobianToRestart = false;
        % calculate the change
        dx = dxTrial;
        df = fvalTrial - fval;
        % safeguard threshold to avoid too small steps: no more than the numerical derivative step
        dxThreshold = max(FinDiffRelStep.*abs(x),eps);
        % check convergence using other criteria
        metric_dx = max(abs(dx.*xScale)./(abs(x.*xScale) + 1.0));
        if  metric_dx <= tolX
            exitflag = 2;
            [x,fval,output] = outputfcn(x,fval,iter,funcCount,algorithm,fjac,fjacinv,xScale,fvalScale);
            return;
        end
        % update to the new value
        x = xTrial;
        fval = fvalTrial;
        fnorm = norm(fval,normOrder);
        fnorm2 = fnorm2Trial;
    end    
end

% the algorithm failed
exitflag = 0;
[x,fval,output] = outputfcn(x,fval,iter,funcCount,algorithm,fjac,fjacinv,xScale,fvalScale);

end

%------------------------------------------------------------------------------
% Subfunction: Generate Output Solution
%------------------------------------------------------------------------------
function [x,fval,output] = outputfcn(x,fval,iter,funcCount,algorithm,fjac,fjacinv,xScale,fvalScale)
x = x.*xScale;
fval = fval.*fvalScale; 
output.iterations = iter;
output.funcCount = funcCount;
output.algorithm = algorithm;
if ~isempty(fjac) % only multiply when nonempty
    % equivalent to: fjac*diag(1./xScale)
    output.Jacobian = bsxfun(@times,fjac,1.0./xScale(:).');
else     
    output.Jacobian = [];    
end
if ~isempty(fjacinv) % only multiply when nonempty
    % equivalent to: diag(xScale)*fjacinv
    output.JacobianInv = bsxfun(@times,xScale(:),fjacinv);
else     
    output.JacobianInv = [];    
end
 
end
%------------------------------------------------------------------------------
% Subfunction: Forward Difference Numerical Jacobian
%------------------------------------------------------------------------------
function fjac = fdjac1(f,x0,f0,FinDiffRelStep)

if isempty(FinDiffRelStep)
    tol = sqrt(eps);
else
    tol = FinDiffRelStep;    
end

% h  = tol.*max(abs(x0),1);
h  = max(tol.*abs(x0),eps);
xh = x0 + h;
h  = xh - x0; % reduce roundoff error: see Numerical Recipe (3rd Edition, Equation 5.7.4)
nx = length(x0);

fjac = zeros(size(f0,1),nx);       % Jacobian matrix
for ii = 1:nx
   xx = x0; 
   xx(ii) = xh(ii); 
   f1 = f(xx);
   fjac(:,ii) = (f1 - f0)/h(ii);
end

end % end function fdjac1

%------------------------------------------------------------------------------
% Subfunction: three-point safeguarded parabolic model for a line search
%------------------------------------------------------------------------------
function lambdap = parab3p(lambdac, lambdam, ff0, ffc, ffm,safeGuard_low,safeGuard_high)
% Apply three-point safeguarded parabolic model for a line search.
%
% C. T. Kelley, June 29, 1994
%
% This code comes with no guarantee or warranty of any kind.
%
% function lambdap = parab3p(lambdac, lambdam, ff0, ffc, ffm)
%
% input:
%       lambdac = current steplength
%       lambdam = previous steplength
%       ff0 = value of \| F(x_c) \|^2
%       ffc = value of \| F(x_c + \lambdac d) \|^2
%       ffm = value of \| F(x_c + \lambdam d) \|^2
%
% output:
%       lambdap = new value of lambda given parabolic model
%
% internal parameters:
%       sigma0 = .1, sigma1 = .5, safeguarding bounds for the linesearch
%

%
% Set internal parameters.
%
sigma0 = safeGuard_low; sigma1 = safeGuard_high;
%
% Compute coefficients of interpolation polynomial.
%
% p(lambda) = ff0 + (c1 lambda + c2 lambda^2)/d1
%
% d1 = (lambdac - lambdam)*lambdac*lambdam < 0
%      so, if c2 > 0 we have negative curvature and default to
%      lambdap = sigam1 * lambda.
%
c2 = lambdam*(ffc-ff0)-lambdac*(ffm-ff0);
if c2 >= 0
    lambdap = sigma1*lambdac; return
end
c1 = lambdac*lambdac*(ffm-ff0)-lambdam*lambdam*(ffc-ff0);
lambdap = -c1*.5/c2;
if lambdap < sigma0*lambdac, lambdap = sigma0*lambdac; end
if lambdap > sigma1*lambdac, lambdap = sigma1*lambdac; end
end