function transAll = transmat1d(trans_s,policy,grid)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Purpose: compute the transition matrix of all state variables from
%          exogenous Probality Transition(trans_s) and endogenous transition(policy)
% where
%       trans_s  : (S by S) exogenous state variable transition matrix, (trans_s(i,j)=trans_s(t+1=j|t=i))
%       policy: (ngrid by S) matrix of (i)discrete state policy index from "max" 
%               if "nargin ==2"; (ii) continuous state policy fcn if "nargin ==3"
%       grid  : (ngrid by 1) uniform grid values of the endogenous state variable
%       transAll : (ngrid*S) by (ngrid*S) SPARSE transition matrix, the order
%               of vectorization is the same as the memory order of policy 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Jinhui Bai (jinhui.bai@gmail.com); 
% History: First: Nov 11, 2004 
%          Revised: January 3, 2005; Sept 29, 2005; 
%          June 15, 2015: allow the possibility of non-uniform grid
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% error check
[rowProb,colProb] = size(trans_s);
if ~isequal(rowProb,colProb)
    error('The first input must be a square matrix');
end
S = rowProb; % the number of exogenous shocks 

% find out the order of transAll: it follows the memory order of policy itself
% Note: if isequal(S,nGrid) so that the size does not reveal the order, we assume that s is on the row dimension of policy
[rowPolicy,colPolicy] = size(policy);
if isequal(rowPolicy,S)
    is_first_s = true;
    nGrid = colPolicy;
elseif isequal(colPolicy,S)
    is_first_s = false;
    nGrid = rowPolicy;
else 
    error('The column of the first and second input must agree');
end

% check the grid vector itself
if nargin == 3
    if ~isvector(grid)
        error('The third input must be a vector');
    end
    if ~isequal(length(grid),nGrid)
        error('The row of the second and third input must agree');
    end
end

%
% The main part
%
transAll = zeros(S*nGrid,S*nGrid);      % transition for all states
sVec = (1:S).';

if isequal(nargin,2)
    for s = 1:S % for today's shock
        for i = 1:nGrid % for today's grid point
            if is_first_s % s comes first
                idx_now = (i-1)*S + s; 
                idx_next = (policy(s,i) - 1)*S + sVec;            
            else % grid comes first
                idx_now = (s-1)*nGrid + i;
                idx_next = (sVec - 1)*nGrid + policy(i,s);            
            end
            transAll(idx_now,idx_next) = trans_s(s,:);
        end
    end    
elseif isequal(nargin,3)
    [~,~,idx_policy_grid] = histcounts(policy(:).',[-inf,grid(:).',inf]);
    idx_policy = reshape(idx_policy_grid,size(policy));
    for s = 1:S % for today's shock
        for i = 1:nGrid % for today's grid point
            if is_first_s % s comes first
                idx_now = (i-1)*S + s;
                indexLeft = idx_policy(s,i) - 1;
                policy_temp = policy(s,i);
            else % grid comes first
                idx_now = (s-1)*nGrid + i;
                indexLeft = idx_policy(i,s) - 1;
                policy_temp = policy(i,s);
            end            
            indexRight = indexLeft + 1;                        % Right grid point
            if (indexLeft >= 1) && (indexRight <= nGrid)                
                prob_Left  = (grid(indexRight) - policy_temp)/(grid(indexRight) - grid(indexLeft));  % split trans_s to left in a reverse way
                prob_Right = 1.0 - prob_Left;  % split trans_s to right in a reverse way
                if is_first_s % s comes first                    
                    idx_next = [(indexLeft - 1)*S + sVec(:).',(indexRight - 1)*S + sVec(:).'];
                else % grid comes first                    
                    idx_next = [(sVec(:).' - 1)*nGrid + indexLeft,(sVec(:).' - 1)*nGrid + indexRight];
                end
                transAll(idx_now,idx_next) = [trans_s(s,:)*prob_Left,trans_s(s,:)*prob_Right];
            elseif indexLeft < 1 % give the prob to the leftmost point if overshoot left
                if is_first_s % s comes first
                    idx_next = sVec(:).';
                else % grid comes first
                    idx_next = (sVec(:).' - 1)*nGrid + 1;
                end
                transAll(idx_now,idx_next) = trans_s(s,:);
            elseif (indexRight > nGrid)  % give the prob to the rightmost point if overshoot right
                if is_first_s % s comes first
                    idx_next = (nGrid - 1)*S + sVec;
                else % grid comes first
                    idx_next = (sVec - 1)*nGrid + nGrid;
                end                
                transAll(idx_now,idx_next) = trans_s(s,:);
            end
        end
    end
end
